"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchTransfers = void 0;
const wormhole_1 = require("./wormhole");
const logger_1 = require("../logger");
const fetchTransfers = (bridgeProgram, 
// @ts-ignore
erc20Contract, ethTokenBridge, fromBlock, f, log = logger_1.consoleLogger) => __awaiter(void 0, void 0, void 0, function* () {
    const { ethKey, solKey } = f;
    const transfers = [];
    if (ethKey) {
        const events = yield erc20Contract.getPastEvents('Transfer', {
            filter: {
                from: ethKey,
                to: ethTokenBridge.toLowerCase(),
            },
            fromBlock,
        }, (err, events) => {
            throw err;
        });
        const accts = yield bridgeProgram.account.transferV2.all([
            {
                memcmp: {
                    offset: 8,
                    bytes: solKey,
                },
            },
        ]);
        log.info(`ethKey ${ethKey} -> ${events.length} eth burns`);
        log.info(`solKey ${solKey} -> ${accts.length} sol transfer pdas`);
        const fetches = [];
        const acctsBySequence = {};
        for (const acct of accts) {
            // @ts-ignore
            log.info(`transfer ${acct.publicKey.toString()} pda ${JSON.stringify(Object.assign(Object.assign({}, acct.account), { createdAt: acct.account.createdAt, incentiveEpoch: acct.account.epochId }))}`);
            // @ts-ignore
            if (!acct.account.sequence) {
                // legacy transfers
                continue;
            }
            // @ts-ignore
            const seq = acct.account.sequence.toNumber();
            fetches.push((0, wormhole_1.fetchEthInfo)(seq, ethTokenBridge));
            acctsBySequence[seq] = acct;
        }
        const reses = yield Promise.all(fetches);
        const transferPDAByEthTx = {};
        for (const [seq, { originTx: { txHash }, },] of reses) {
            transferPDAByEthTx[txHash] = acctsBySequence[seq];
        }
        for (const event of events.reverse()) {
            const pda = transferPDAByEthTx[event.transactionHash];
            console.log(`ethTx ${event.transactionHash} -> ${JSON.stringify(pda)}`);
            transfers.push({
                ethTx: event.transactionHash,
                blockNo: event.blockNumber,
                pda: !pda
                    ? undefined
                    : {
                        key: pda.publicKey.toString(),
                        createdAt: pda.account.createdAt.toNumber(),
                        amount: pda.account.amount.toNumber(),
                        incentiveRound: pda.account.epochId
                            ? pda.account.epochId.toNumber()
                            : undefined,
                        points: pda.account.points
                            ? pda.account.points.toString()
                            : undefined,
                    },
            });
        }
    }
    else {
        const accts = yield bridgeProgram.account.transferV2.all([
            {
                memcmp: {
                    offset: 8,
                    bytes: solKey,
                },
            },
        ]);
        for (const { account, publicKey } of accts) {
            transfers.push({
                pda: {
                    key: publicKey.toString(),
                    createdAt: account.createdAt.toNumber(),
                    amount: account.amount.toNumber(),
                    incentiveRound: account.epochId
                        ? account.epochId.toNumber()
                        : undefined,
                    points: account.points ? account.points.toString() : undefined,
                },
            });
        }
    }
    log.debug(`transfers ${JSON.stringify(transfers)}`);
    return transfers;
});
exports.fetchTransfers = fetchTransfers;
//# sourceMappingURL=onchain.js.map