"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Deferred_promise, _Deferred_resolve, _Deferred_reject;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deferred = exports.pgClient = exports.BURN_TABLE = exports.JOB_TABLE = exports.JOB_ID_TABLE = exports.NODE_OPERATOR_EPOCH_TABLE = exports.SQUADS_TX_TABLE = exports.UPGRADE_REWARDS_TRANSFER_TABLE = exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE = exports.NODE_OPERATOR_REWARDS_TRANSFER_TABLE = exports.TRANSFER_TABLE = void 0;
const pg_1 = require("pg");
const kysely_1 = require("kysely");
const logger_1 = require("../../logger");
const transferFromDB = (o) => {
    return {
        id: o.id,
        solKey: o.sol_key,
        solTx: o.sol_tx,
        sqTx: o.sq_tx,
        createdAt: o.created_at,
        payedAt: o.payed_at,
        amountDue: BigInt(o.amount_due),
        amountPayed: o.amount_payed ? BigInt(o.amount_payed) : undefined,
        operatorRewards: o.transfer_type == 'node_operator_rewards'
            ? {
                epochId: o.epoch_id,
                startTime: o.start_time,
                endTime: o.end_time,
            }
            : undefined,
        computeClientRewards: o.transfer_type == 'open_compute_rewards'
            ? {
                epochId: o.epoch_id,
                deviceId: o.device_id,
            }
            : undefined,
        upgradeRewards: o.transfer_type == 'upgrade_rewards' && !!o.epoch_id
            ? {
                epochId: o.epoch_id,
                epochStartTime: o.epoch_start_time,
                points: BigInt(o.points),
            }
            : undefined,
    };
};
const txHandle = (ps, log = logger_1.consoleLogger) => {
    let { trx, commit, rollback } = ps;
    return {
        trx,
        commit,
        rollback,
        /*async releaseLock(f: { lock: Lock; newCursor?: string }): Promise<void> {
          const now = new Date(Date.now())
          const res = await trx
            .updateTable(LOCK_TABLE)
            .set({
              ...(f.newCursor ? { cursor: f.newCursor } : {}),
              released_at: now,
            })
            .where('description', '=', f.lock)
            .executeTakeFirstOrThrow()
          log.debug(`updated ${res.numUpdatedRows} lock`)
          commit()
        },*/
    };
};
exports.TRANSFER_TABLE = 'transfer';
exports.NODE_OPERATOR_REWARDS_TRANSFER_TABLE = 'node_operator_rewards_transfer';
exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE = 'open_compute_rewards_transfer';
exports.UPGRADE_REWARDS_TRANSFER_TABLE = 'upgrade_rewards_transfer';
exports.SQUADS_TX_TABLE = 'squads_tx';
exports.NODE_OPERATOR_EPOCH_TABLE = 'node_operator_epoch';
exports.JOB_ID_TABLE = 'job_id';
exports.JOB_TABLE = 'job';
exports.BURN_TABLE = 'burn';
const pgClient = (config) => {
    var _a, _b;
    const dialect = new kysely_1.PostgresDialect({
        pool: new pg_1.Pool(Object.assign(Object.assign({}, config), { port: (_a = config.port) !== null && _a !== void 0 ? _a : 5432, max: (_b = config.max) !== null && _b !== void 0 ? _b : 10 })),
    });
    // Database interface is passed to Kysely's constructor, and from now on, Kysely
    // knows your database structure.
    // Dialect is passed to Kysely's constructor, and from now on, Kysely knows how
    // to communicate with your database.
    const db = new kysely_1.Kysely({
        dialect,
    });
    const createTransfers = (db, transfers, log = logger_1.consoleLogger) => __awaiter(void 0, void 0, void 0, function* () {
        for (const t of transfers) {
            const { id: transferId } = (yield db
                .insertInto(exports.TRANSFER_TABLE)
                .values({
                amount_due: BigInt(t.amountDue),
                amount_payed: t.amountPayed ? BigInt(t.amountPayed) : 0,
                created_at: t.createdAt,
                payed_at: t.payedAt,
                sol_key: t.solKey,
                sol_tx: t.solTx,
                transfer_type: (() => {
                    if (t.operatorRewards) {
                        return 'node_operator_rewards';
                    }
                    else if (t.upgradeRewards) {
                        return 'upgrade_rewards';
                    }
                    else if (t.computeClientRewards) {
                        return 'open_compute_rewards';
                    }
                    throw new Error('wtf');
                })(),
            })
                .returning((eb) => ['id as id'])
                .executeTakeFirst());
            log.info(`inserted transfer ${JSON.stringify(t, (key, value) => typeof value === 'bigint' ? value.toString() : value)}`);
            if (t.operatorRewards) {
                log.info(`inserting operator rewards ${transferId}`);
                const x = (yield db
                    .insertInto(exports.NODE_OPERATOR_REWARDS_TRANSFER_TABLE)
                    .values({
                    end_time: t.operatorRewards.endTime,
                    start_time: t.operatorRewards.startTime,
                    epoch_id: t.operatorRewards.epochId,
                    transfer_id: transferId,
                })
                    .executeTakeFirst()).insertId;
            }
            else if (t.computeClientRewards) {
                log.info(`inserting compute client rewards ${transferId}`);
                const x = (yield db
                    .insertInto(exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE)
                    .values({
                    epoch_id: t.computeClientRewards.epochId,
                    device_id: t.computeClientRewards.deviceId,
                    transfer_id: transferId,
                })
                    .executeTakeFirst()).insertId;
            }
            else if (t.upgradeRewards) {
                log.info(`inserting upgrade rewards ${transferId}`);
                const x = (yield db
                    .insertInto(exports.UPGRADE_REWARDS_TRANSFER_TABLE)
                    .values({
                    epoch_id: t.upgradeRewards.epochId,
                    epoch_start_time: t.upgradeRewards.epochStartTime,
                    points: t.upgradeRewards.points,
                    transfer_id: transferId,
                })
                    .executeTakeFirst()).insertId;
            }
        }
        return;
    });
    return {
        db,
        saveNodeOperatorEpoch(epochId, transfers, jobIds, totalRndr, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                yield db.transaction().execute((trx) => __awaiter(this, void 0, void 0, function* () {
                    yield createTransfers(trx, transfers, log);
                    log.info(`inserted transfers`);
                    for (const id of jobIds) {
                        yield trx
                            .insertInto(exports.JOB_ID_TABLE)
                            .values({
                            id: BigInt(id),
                            epoch_id: epochId,
                        })
                            .executeTakeFirstOrThrow();
                    }
                    log.info(`inserted jobids`);
                    const now = new Date(Date.now());
                    const res = yield trx
                        .updateTable(exports.NODE_OPERATOR_EPOCH_TABLE)
                        .set({
                        saved_at: now,
                        total_rndr: totalRndr,
                    })
                        .where('id', '=', epochId.toString())
                        .executeTakeFirstOrThrow();
                    log.info(`saved node operator epoch: ${res.numUpdatedRows}`);
                }));
                return;
            });
        },
        getLatestNodeOperatorEpoch(log = logger_1.consoleLogger) {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                const now = new Date(Date.now());
                try {
                    const res = yield db
                        .selectFrom(exports.NODE_OPERATOR_EPOCH_TABLE)
                        .selectAll()
                        .where('end_time', '<=', now)
                        .where('saved_at', 'is', null)
                        .orderBy('end_time asc')
                        .limit(1)
                        .executeTakeFirstOrThrow();
                    return {
                        epochId: BigInt(res.id),
                        startedAt: res.start_time,
                        endedAt: res.end_time,
                        savedAt: (_a = res.saved_at) !== null && _a !== void 0 ? _a : undefined,
                    };
                }
                catch (e) {
                    if (e instanceof kysely_1.NoResultError) {
                        log.info(`no node operator epoch found`);
                        return undefined;
                    }
                    throw e;
                }
            });
        },
        getOutstanding(f, log = logger_1.consoleLogger) {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                const { typ, nullKey, due } = f;
                //select sol_key, sum(amount_due)-sum(amount_payed) from transfer group by sol_key;
                let q = db
                    .selectFrom(exports.TRANSFER_TABLE)
                    .select(({ fn, val, ref }) => [
                    'sol_key',
                    (0, kysely_1.sql) `${fn.sum('amount_due')} -
          ${fn.sum('amount_payed')}`.as('due'),
                ])
                    .where('transfer_type', '=', typ)
                    .groupBy(['sol_key']);
                if (!nullKey) {
                    q = q.where('sol_key', 'is not', null);
                }
                if (due) {
                    q = q.having(({ fn, ref }) => (0, kysely_1.sql) `${fn.sum('amount_due')} -
            ${fn.sum('amount_payed')}
            >
            0`);
                }
                const res = yield q.execute();
                log.info(`outstanding query ${q.compile().sql} params ${JSON.stringify(f)} ${res.length} rows`);
                const out = {};
                for (const r of res) {
                    out[(_a = r.sol_key) !== null && _a !== void 0 ? _a : ''] = BigInt(r.due);
                }
                return out;
            });
        },
        fetchTransfers(f, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                let base = db
                    .selectFrom(exports.TRANSFER_TABLE)
                    // @ts-ignore
                    .selectAll(exports.TRANSFER_TABLE)
                    .where('transfer.transfer_type', '=', f.typ);
                if (f.solKey) {
                    base = base.where('transfer.sol_key', '=', f.solKey);
                }
                if (f.payed != undefined) {
                    base = base.where('transfer.payed_at', f.payed ? 'is not' : 'is', null);
                }
                if (f.createdBefore != undefined) {
                    base = base.where('transfer.created_at', '<=', f.createdBefore);
                }
                if (f.typ == 'upgrade_rewards') {
                    base = base
                        .innerJoin(exports.UPGRADE_REWARDS_TRANSFER_TABLE, 'upgrade_rewards_transfer.transfer_id', 'transfer.id')
                        .selectAll(exports.UPGRADE_REWARDS_TRANSFER_TABLE);
                    if (f.epochId) {
                        // @ts-ignore
                        base = base.where('upgrade_rewards_transfer.epoch_id', '=', f.epochId);
                    }
                }
                if (f.typ == 'open_compute_rewards') {
                    base = base
                        .innerJoin(exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE, 'open_compute_rewards_transfer.transfer_id', 'transfer.id')
                        .selectAll(exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE);
                    if (f.epochId) {
                        base = base.where(
                        // @ts-ignore
                        'open_compute_rewards_transfer.epoch_id', '=', f.epochId);
                    }
                }
                if (f.typ == 'node_operator_rewards') {
                    base = base
                        .innerJoin(exports.NODE_OPERATOR_REWARDS_TRANSFER_TABLE, 'node_operator_rewards_transfer.transfer_id', 'transfer.id')
                        .selectAll(exports.NODE_OPERATOR_REWARDS_TRANSFER_TABLE);
                    if (f.epochId) {
                        base = base.where(
                        // @ts-ignore
                        'node_operator_rewards_transfer.epoch_id', '=', f.epochId);
                    }
                }
                if (f.sort) {
                    base = base.orderBy('transfer.sol_key asc');
                }
                const res = yield base.execute();
                log.info(`fetched ${base.compile().sql} params ${JSON.stringify(f)} ${res.length} transfers`);
                return res.map((r) => transferFromDB(r));
            });
        },
        payoutTransfers(idToAmt, payedAt, solTx, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield db.transaction().execute((tx) => __awaiter(this, void 0, void 0, function* () {
                    const alreadyUpdatedIds = (yield tx
                        .selectFrom(exports.TRANSFER_TABLE)
                        .select(['id'])
                        .where('id', 'in', Object.keys(idToAmt))
                        .where('sol_tx', 'is not', null)
                        .execute()).map(({ id }) => id);
                    for (const [id, amt] of Object.entries(idToAmt)) {
                        if (alreadyUpdatedIds.includes(id)) {
                            log.info(`transfer ${id} already marked payed out. skipping`);
                            continue;
                        }
                        const res = yield tx
                            .updateTable(exports.TRANSFER_TABLE)
                            .set({
                            payed_at: payedAt,
                            sol_tx: solTx,
                            amount_payed: amt,
                        })
                            .where('id', '=', id.toString())
                            .executeTakeFirstOrThrow();
                        log.info(`marked ${res.numUpdatedRows} transfer payed out`);
                    }
                }));
            });
        },
        payoutTransfer(id, payedAt, solTx, amtPayed, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield db
                    .updateTable(exports.TRANSFER_TABLE)
                    .set({
                    payed_at: payedAt,
                    sol_tx: solTx,
                    amount_payed: amtPayed,
                })
                    .where('id', '=', id.toString())
                    .executeTakeFirst();
                log.info(`marked ${res.numUpdatedRows} transfer payed out`);
                return;
            });
        },
        createTransfers(transfers, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                yield createTransfers(db, transfers, log);
            });
        },
        fetchInfoForEpoch(epochId) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield db
                    .selectFrom(exports.JOB_ID_TABLE)
                    .innerJoin(exports.NODE_OPERATOR_EPOCH_TABLE, 'node_operator_epoch.id', 'job_id.epoch_id')
                    .where('job_id.epoch_id', '=', epochId.toString())
                    .select([
                    'job_id.id',
                    'node_operator_epoch.total_rndr',
                    'node_operator_epoch.start_time',
                    'node_operator_epoch.end_time',
                    'node_operator_epoch.burned',
                    'node_operator_epoch.burn_tx',
                ])
                    .execute();
                if (res.length == 0) {
                    throw new Error(`epoch ${epochId} not found`);
                }
                return {
                    jobIds: res.map((r) => Number(r.id)),
                    totalRndr: BigInt(res[0].total_rndr),
                    startTime: res[0].start_time,
                    endTime: res[0].end_time,
                    burned: BigInt(res[0].burned),
                    burnTx: res[0].burn_tx,
                };
            });
        },
        updateComputeClientSolKey(deviceId, solKey, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const q = db
                    .updateTable(`${exports.TRANSFER_TABLE} as t`)
                    .set({
                    sol_key: solKey,
                })
                    .from(`${exports.OPEN_COMPUTE_REWARDS_TRANSFER_TABLE} as o`)
                    .whereRef('t.id', '=', 'o.transfer_id')
                    .where('t.sol_key', 'is', null)
                    .where('o.device_id', '=', deviceId);
                const sql = yield q.compile();
                log.info(`sql ${sql.sql} params ${sql.parameters}`);
                const res = yield q.executeTakeFirstOrThrow();
                log.info(`updated ${res.numUpdatedRows} device-id sol_keys`);
            });
        },
        markJobsBurned(p, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                let { markedAt, solTx } = p;
                let res = yield db
                    .updateTable(exports.BURN_TABLE)
                    .set({ marked_burned_at: markedAt })
                    .where('sol_tx', '=', solTx)
                    .executeTakeFirstOrThrow();
                log.debug(`set ${res.numUpdatedRows} ${solTx} -> marked_burned_at ${markedAt.toISOString()}`);
            });
        },
        markJobsBurnt(trx, p, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const { id } = yield trx
                    .insertInto(exports.BURN_TABLE)
                    .values({
                    sol_tx: p.solTx,
                    executed_at: p.executedAt,
                    priced_at: p.pricedAt,
                    usdc_spent: p.usdcSpent,
                    quote_amt: p.quoteAmt,
                    burned: p.burned,
                })
                    .onConflict((oc) => oc.column('sol_tx').doNothing())
                    .returning((eb) => ['id as id'])
                    .executeTakeFirstOrThrow();
                const jobIdsStr = p.jobsIds.map(String);
                const res = yield trx
                    .updateTable(exports.JOB_TABLE)
                    .where('id', 'in', jobIdsStr)
                    .set('burn_id', id)
                    .executeTakeFirstOrThrow();
                log.debug(`set burn_id on ${res.numUpdatedRows} jobs`);
                return;
            });
        },
        produceBurnedJobs(jobs, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                yield db.transaction().execute((trx) => __awaiter(this, void 0, void 0, function* () {
                    var _a;
                    let num = BigInt(0);
                    for (const { amount, completedAt, jobId } of jobs) {
                        const res = yield trx
                            .insertInto(exports.JOB_TABLE)
                            .values({
                            id: jobId,
                            render_amt: amount,
                            completed_at: completedAt,
                            inserted_at: new Date(Date.now()),
                        })
                            .onConflict((oc) => oc.column('id').doNothing())
                            .executeTakeFirstOrThrow();
                        num += (_a = res.numInsertedOrUpdatedRows) !== null && _a !== void 0 ? _a : BigInt(0);
                    }
                    log.debug(`inserted ${num} of ${jobs.length} burn jobs`);
                }));
            });
        },
        consumeAndLockBurnJobs(limit, lowerRndrThreshold, upperRndrThreshold, log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const trx = yield begin(db);
                const q = trx.trx
                    .selectFrom(trx.trx
                    .selectFrom(exports.JOB_TABLE)
                    .select(({ fn, val, ref }) => [
                    'id',
                    fn
                        .sum('render_amt')
                        .over((ob) => ob.orderBy(`completed_at`, `asc`))
                        .as('total'),
                ])
                    .where('burn_id', 'is', null)
                    .as('t'))
                    .select('id')
                    .where('total', '>', lowerRndrThreshold)
                    .where('total', '<', upperRndrThreshold);
                log.debug(`outstanding jobs q: ${q.compile().sql} ${JSON.stringify({
                    limit,
                    lowerRndrThreshold,
                    upperRndrThreshold,
                })}`);
                const idRs = yield q.execute();
                const ids = idRs.map((x) => x.id);
                log.debug(`${ids.length} candidate jobs for burn. locking up to ${limit} jobs`);
                if (ids.length == 0) {
                    return { trx, jobs: [] };
                }
                const rows = yield trx.trx
                    .selectFrom(exports.JOB_TABLE)
                    .selectAll()
                    .where('id', 'in', ids)
                    .limit(limit)
                    .forUpdate()
                    .skipLocked()
                    .execute();
                log.debug(`locked ${rows.length} jobs`);
                const jobs = [];
                for (const r of rows) {
                    jobs.push({
                        id: Number(r.id),
                        completedAt: r.completed_at,
                        rndrUsed: BigInt(r.render_amt),
                    });
                }
                return { trx, jobs };
            });
        },
        incrBuyBurnMetrics(log = logger_1.consoleLogger) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield db
                    .selectFrom(exports.JOB_TABLE)
                    .select('completed_at')
                    .orderBy('completed_at asc')
                    .where('burn_id', 'is', null)
                    .execute();
                if (res.length == 0) {
                    return { outstandingJobs: 0 };
                }
                return {
                    outstandingJobs: res.length,
                    oldestJobFinishedAt: res[0].completed_at,
                };
            });
        },
        deleteAll() {
            return __awaiter(this, void 0, void 0, function* () { });
        },
    };
};
exports.pgClient = pgClient;
function begin(db) {
    return __awaiter(this, void 0, void 0, function* () {
        const connection = new Deferred();
        const result = new Deferred();
        // Do NOT await this line.
        db.transaction()
            .execute((trx) => {
            connection.resolve(trx);
            return result.promise;
        })
            .catch((err) => {
            // Don't do anything here. Just swallow the exception.
        });
        const trx = yield connection.promise;
        return {
            trx,
            commit() {
                result.resolve(null);
            },
            rollback() {
                result.reject(new Error('rollback'));
            },
        };
    });
}
class Deferred {
    constructor() {
        _Deferred_promise.set(this, void 0);
        _Deferred_resolve.set(this, void 0);
        _Deferred_reject.set(this, void 0);
        this.resolve = (value) => {
            if (__classPrivateFieldGet(this, _Deferred_resolve, "f")) {
                __classPrivateFieldGet(this, _Deferred_resolve, "f").call(this, value);
            }
        };
        this.reject = (reason) => {
            if (__classPrivateFieldGet(this, _Deferred_reject, "f")) {
                __classPrivateFieldGet(this, _Deferred_reject, "f").call(this, reason);
            }
        };
        __classPrivateFieldSet(this, _Deferred_promise, new Promise((resolve, reject) => {
            __classPrivateFieldSet(this, _Deferred_reject, reject, "f");
            __classPrivateFieldSet(this, _Deferred_resolve, resolve, "f");
        }), "f");
    }
    get promise() {
        return __classPrivateFieldGet(this, _Deferred_promise, "f");
    }
}
exports.Deferred = Deferred;
_Deferred_promise = new WeakMap(), _Deferred_resolve = new WeakMap(), _Deferred_reject = new WeakMap();
//# sourceMappingURL=v1.js.map