import { fetchEthInfo } from './wormhole';
import { consoleLogger } from '../logger';
export const fetchTransfers = async (bridgeProgram, 
// @ts-ignore
erc20Contract, ethTokenBridge, fromBlock, f, log = consoleLogger) => {
    const { ethKey, solKey } = f;
    const transfers = [];
    if (ethKey) {
        const events = await erc20Contract.getPastEvents('Transfer', {
            filter: {
                from: ethKey,
                to: ethTokenBridge.toLowerCase(),
            },
            fromBlock,
        }, (err, events) => {
            throw err;
        });
        const accts = await bridgeProgram.account.transferV2.all([
            {
                memcmp: {
                    offset: 8,
                    bytes: solKey,
                },
            },
        ]);
        log.info(`ethKey ${ethKey} -> ${events.length} eth burns`);
        log.info(`solKey ${solKey} -> ${accts.length} sol transfer pdas`);
        const fetches = [];
        const acctsBySequence = {};
        for (const acct of accts) {
            // @ts-ignore
            log.info(`transfer ${acct.publicKey.toString()} pda ${JSON.stringify({
                ...acct.account,
                createdAt: acct.account.createdAt,
                incentiveEpoch: acct.account.epochId,
            })}`);
            // @ts-ignore
            if (!acct.account.sequence) {
                // legacy transfers
                continue;
            }
            // @ts-ignore
            const seq = acct.account.sequence.toNumber();
            fetches.push(fetchEthInfo(seq, ethTokenBridge));
            acctsBySequence[seq] = acct;
        }
        const reses = await Promise.all(fetches);
        const transferPDAByEthTx = {};
        for (const [seq, { originTx: { txHash }, },] of reses) {
            transferPDAByEthTx[txHash] = acctsBySequence[seq];
        }
        for (const event of events.reverse()) {
            const pda = transferPDAByEthTx[event.transactionHash];
            console.log(`ethTx ${event.transactionHash} -> ${JSON.stringify(pda)}`);
            transfers.push({
                ethTx: event.transactionHash,
                blockNo: event.blockNumber,
                pda: !pda
                    ? undefined
                    : {
                        key: pda.publicKey.toString(),
                        createdAt: pda.account.createdAt.toNumber(),
                        amount: pda.account.amount.toNumber(),
                        incentiveRound: pda.account.epochId
                            ? pda.account.epochId.toNumber()
                            : undefined,
                        points: pda.account.points
                            ? pda.account.points.toString()
                            : undefined,
                    },
            });
        }
    }
    else {
        const accts = await bridgeProgram.account.transferV2.all([
            {
                memcmp: {
                    offset: 8,
                    bytes: solKey,
                },
            },
        ]);
        for (const { account, publicKey } of accts) {
            transfers.push({
                pda: {
                    key: publicKey.toString(),
                    createdAt: account.createdAt.toNumber(),
                    amount: account.amount.toNumber(),
                    incentiveRound: account.epochId
                        ? account.epochId.toNumber()
                        : undefined,
                    points: account.points ? account.points.toString() : undefined,
                },
            });
        }
    }
    log.debug(`transfers ${JSON.stringify(transfers)}`);
    return transfers;
};
//# sourceMappingURL=onchain.js.map