import type { ColumnType } from "kysely";
export type Channel = "availability" | "node_operator" | "open_compute" | "upgrade";
export type Generated<T> = T extends ColumnType<infer S, infer I, infer U> ? ColumnType<S, I | undefined, U> : ColumnType<T, T | undefined, T>;
export type Int8 = ColumnType<string, bigint | number | string, bigint | number | string>;
export type Timestamp = ColumnType<Date, Date | string, Date | string>;
export interface Burn {
    burned: Int8;
    id: Generated<Int8>;
    marked_burned_at: Timestamp | null;
    priced_at: Timestamp | null;
    quote_amt: Int8 | null;
    sol_tx_id: Generated<Int8>;
    usdc_spent: Int8 | null;
}
export interface Entity {
    id: Generated<Int8>;
    sol_key: string | null;
}
export interface Epoch {
    channel: Channel | null;
    end_time: Timestamp | null;
    id: Generated<Int8>;
    saved_at: Timestamp | null;
    start_time: Timestamp | null;
    total_rndr: Generated<Int8 | null>;
    ui: Generated<Int8>;
}
export interface Job {
    burn_id: Generated<Int8>;
    completed_at: Timestamp | null;
    id: Generated<Int8>;
    inserted_at: Generated<Timestamp | null>;
    render_amt: Int8;
}
export interface JobId {
    epoch_id: Generated<Int8>;
    id: Int8;
}
export interface ManualBurn {
    burned: Int8;
    epoch_id: Generated<Int8>;
    total_rndr: Int8;
    tx_sig: string;
}
export interface SolTransfer {
    amount_due: Int8;
    amount_payed: Int8;
    channel: Channel;
    created_at: Generated<Timestamp>;
    entity_id: Generated<Int8>;
    id: Generated<Int8>;
    sol_tx_id: Generated<Int8>;
}
export interface SolTransferInfo {
    device_id: string | null;
    epoch_id: Generated<Int8>;
    points: Int8 | null;
    transfer_id: Generated<Int8>;
}
export interface SolTx {
    executed_at: Timestamp | null;
    id: Generated<Int8>;
    is_sq: Generated<boolean | null>;
    sig: string;
}
export interface DB {
    burn: Burn;
    entity: Entity;
    epoch: Epoch;
    job: Job;
    job_id: JobId;
    manual_burn: ManualBurn;
    sol_transfer: SolTransfer;
    sol_transfer_info: SolTransferInfo;
    sol_tx: SolTx;
}
//# sourceMappingURL=dbTypesV2.d.ts.map